#include "convolution.h"

/* returns position of <x>,<y>'th element in a single dimensional array
 * with <num_col> columns 
*/
int ConvolutionFilter::pos(int x, int y, int num_col){
	return y * num_col + x;
}

/*constructs a new filter of size <_width>, <_height> with <_filter_data>
*/
ConvolutionFilter::ConvolutionFilter(int _width, int _height, float * _filter_data){
	width = _width;
	height = _height;
	filter_data = _filter_data;
}


ConvolutionFilter::ConvolutionFilter(float(*filter_function)(float, float), int _width, int _height){
	width = _width;
	height = _height;
	filter_data = new float[width * height];
	
	int x_rad = width/2;
	int y_rad = height/2;

	for(int i = -x_rad; i <= x_rad; i++){
		for(int j = -y_rad; j <= y_rad; j++){
			filter_data[pos(i + x_rad, j + y_rad, width)] = filter_function(i, x_rad) * filter_function(j, y_rad);
		}
	}
}

/*destructor
*/
ConvolutionFilter::~ConvolutionFilter(){
  if (filter_data)
      delete[] filter_data;
}

/* returns value of filter at position <x>, <y> 
*/
float ConvolutionFilter::getValue(int x, int y){
	if(x < 0 || x >= width || y < 0 || y >= height) {
		return NULL;
	} else {
		return *(filter_data + pos(x, y, width));
	}
}

/* returns convoluted pixel of pixel <x>, <y> of <img>
 */
void ConvolutionFilter::convolutePixel(TargaImage *img, unsigned char *new_pxl, int x, int y){
	new_pxl[0] = new_pxl[1] = new_pxl[2] = new_pxl[3] = 0;
	float f_val;
	double norm = 0;

	int x_rad = width/2;
	int y_rad = height/2;

	unsigned char * pxl;

	//loop over filter
	for(int i = -x_rad; i <= x_rad; i++){
		for(int j = -y_rad; j <= y_rad; j++){
			if((pxl = img->getPixel(x + i, y + j)) != NULL){
				f_val = getValue(i + x_rad, j + y_rad);
				norm += f_val;
				
				//sum averaged weight
				new_pxl[0] += pxl[0] * f_val;
				new_pxl[1] += pxl[1] * f_val;
				new_pxl[2] += pxl[2] * f_val;
			}
		}
	}

	//renormalize
	if(norm != 0){
		new_pxl[0] /= norm;
		new_pxl[1] /= norm;
		new_pxl[2] /= norm;
	}
}

TargaImage* convolute(TargaImage* img, ConvolutionFilter* filter){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * filtered_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	unsigned char * new_pxl;

	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			new_pxl = filtered_image->getPixel(x, y);
			filter->convolutePixel(img, new_pxl, x, y);
		}
	}
	return filtered_image;
}

TargaImage* resample_horz(TargaImage* img, ConvolutionFilter* filter, int new_width){
	return img;
}

TargaImage* resample_vert(TargaImage* img, ConvolutionFilter* filter, int new_height){
	return img;
}

TargaImage* resample_2D(TargaImage* img, ConvolutionFilter* filter, int new_width, int new_height){
	return img;
}

TargaImage* resize(TargaImage* img, ConvolutionFilter* filter, int new_width, int new_height){
	return img;
}