#include "graphics.h"
#include <vector>

/* returns position of <x>,<y>'th element in a single dimensional array
 * with <num_col> columns 
*/
int pos(int x, int y, int num_col){
	return y * num_col + x;
}

/* returns position of <x>,<y>'th element in a single dimensional array
 * with <num_col> columns. Where the center element is indexed as 0,0
 * and negative indexs go back and up.
 */
int c_pos(int x, int y, int rad){
	x = x + rad;
	y = y + rad;
	return pos(x, y, 2*rad+1);
}

/* returns pixel <x>,<y> from <image> or NULL if location is outside of the image

unsigned char* getPixel(TargaImage* image, int x, int y){
	if(x < 0 || x >= image->width || y < 0 || y >= image->height) {
		return NULL;
	} else {
		return image->data + pos(x, y, image->width) * 4;
	}
}*/


/*copies <src> pixel to <dest> pixel
*/
void copy_pxl(unsigned char* src, unsigned char* dest){
	for(int i = 0; i < 4; i++){
		dest[i] = src[i];
	}
}

/* resizes <image> to <new_width>x<new_heigth> pixels using the given <filter>
 * for resampling
 */
TargaImage* resize(TargaImage* image, int new_width, int new_height, float(*filter)(float x, float r)){	
	//create new image for horizontal resize
	unsigned char* horz_data = new unsigned char[new_width * image->height * 4];
	TargaImage* horz_image = new TargaImage(new_width, image->height, horz_data);
	delete[] horz_data;

	//and another for the vertical resize
	unsigned char* vert_data = new unsigned char[new_width * new_height * 4];
	TargaImage* vert_image = new TargaImage(new_width, new_height, vert_data);
	delete[] vert_data;
	
	resample_horz(image, horz_image, filter);
	resample_vert(horz_image, vert_image, filter);

	//remove the useless
	delete horz_image;

	return vert_image;
}

/* samples <new_image> from a reconstructed <image> vertically using the given <filter>
 * for resampling
 */
void resample_vert(TargaImage* image, TargaImage* new_image, float(*filter)(float, float)){	
	float delta_y	= (float)image->height/new_image->height;
	float rad;
	//choose the correct radius
	if(delta_y > 1){
		rad		= delta_y;
	} else {
		rad		= l;
	}

	float y0		= delta_y/2.0 - 0.5;

	//loop over the image
	for(int i = 0; i < image->width; i++){
		for(int j = 0; j < new_image->height; j++){
			unsigned char* new_pxl = new_image->getPixel(i, j);
			new_pxl[0] = new_pxl[1] = new_pxl[2] = new_pxl[3] = 0;
			
			double norm = 0;
			float y = y0 + j * delta_y;

			for(int k = ceil(y - rad); k < floor(y + rad); k++){
				unsigned char* pxl = image->getPixel(i, k);

				if(pxl != NULL){
					float f_val = filter(y - k, rad);
					norm += f_val;
					//sum averaged weight
					new_pxl[0] += pxl[0] * f_val;
					new_pxl[1] += pxl[1] * f_val;
					new_pxl[2] += pxl[2] * f_val;
				}
			}
			//renormalize
			if(norm != 0){
				new_pxl[0] /= norm;
				new_pxl[1] /= norm;
				new_pxl[2] /= norm;
			}
		}
	}
}

/*
*/
void convolute2d_pxl(TargaImage* image, unsigned char* new_pxl, float x, float y, float(*filter)(float, float), int x_rad, int y_rad){
	new_pxl[0] = new_pxl[1] = new_pxl[2] = new_pxl[3] = 0;
	double norm = 0;

	for(int i = ceil(x - x_rad); i <= floor(x + x_rad); i++){
		for(int j = ceil(y - y_rad); j <= floor(y + y_rad); j++){
			unsigned char* pxl = image->getPixel(i, j);

			if(pxl != NULL){
				float f_val = filter(x - i, x_rad) * filter(y - j, y_rad);
				norm += f_val;
				
				//sum averaged weight
				new_pxl[0] += pxl[0] * f_val;
				new_pxl[1] += pxl[1] * f_val;
				new_pxl[2] += pxl[2] * f_val;
			}
		}
	}
	//renormalize
	if(norm != 0){
		new_pxl[0] /= norm;
		new_pxl[1] /= norm;
		new_pxl[2] /= norm;
	}
}

/* samples <new_image> from a reconstructed <image> horizontally using the given <filter>
 * for resampling
 */
void resample_horz(TargaImage* image, TargaImage* new_image, float(*filter)(float, float)){	
	float delta_x	= (float)image->width/new_image->width;
	float rad;
	//choose the correct radius
	if(delta_x > 1){
		rad		= delta_x;
	} else {
		rad		= l;
	}

	float x0		= delta_x/2.0 - 0.5;
	
	//loop over the image
	for(int j = 0; j < image->height; j++){
		for(int i = 0; i < new_image->width; i++){
			unsigned char* new_pxl = new_image->getPixel(i, j);
			new_pxl[0] = new_pxl[1] = new_pxl[2] = new_pxl[3] = 0;
			//printf("new_pxl(%d,%d) ", i, j);
			
			double norm = 0;
			float x = x0 + i * delta_x;

			for(int k = ceil(x - rad); k < floor(x + rad); k++){
				//printf("y:%lf (%d,%d) ", y, i, k);
				unsigned char* pxl = image->getPixel(k, j);

				if(pxl != NULL){
					float f_val = filter(x - k, rad);
					norm += f_val;
					//sum averaged weight
					new_pxl[0] += pxl[0] * f_val;
					new_pxl[1] += pxl[1] * f_val;
					new_pxl[2] += pxl[2] * f_val;
				}
			}
			//renormalize
			if(norm != 0){
				new_pxl[0] /= norm;
				new_pxl[1] /= norm;
				new_pxl[2] /= norm;
			}
		}
	}
}

/* Dithers pixel <x>,<y> of <image> and diffuses the error according to <d_matrix>,
 *  a square matrix of size <d_size>
*/
void FS_dither_pxl(TargaImage* image, int x, int y, float* d_matrix, int d_size){
	float error;
	unsigned char* err_pxl;
	unsigned char* pxl = image->getPixel(x, y);

	//thresh-hold the current pixel
	if(pxl[0] < 128){
		error = (float)pxl[0];
		pxl[0] = 0;
		pxl[1] = 0;
		pxl[2] = 0;

	} else {
		error = (float)pxl[0] - 255;
		pxl[0] = 255;
		pxl[1] = 255;
		pxl[2] = 255;
	}

	//distribute the error
	for(int i = 0; i < d_size; i++){
		for(int j = 0; j < d_size; j++){
			if((err_pxl = image->getPixel(x + (i - d_size/2), y + (j - d_size/2))) != NULL){
				float dis_err = d_matrix[pos(i, j, d_size)] * error;
			
				if(err_pxl[0] + dis_err > 255) {
					err_pxl[0] = 255;
				} else if(err_pxl[0] + dis_err < 0) {
					err_pxl[0] = 0;
				} else {
					err_pxl[0] += (unsigned char)dis_err;
				}
			}
		}
	}
}

TargaImage * moveGlow(TargaImage * img1, TargaImage * img2){
	for(int x = 0; x < img1->width; x++){
		for(int y = 0; y < img1->height; y++){
			unsigned char * pxl1 = img1->getPixel(x, y);
			unsigned char * pxl2 = img2->getPixel(x, y);
			Color c1(pxl1[0], pxl1[1], pxl1[2], 1);
			Color c2(pxl2[0], pxl2[1], pxl2[2], 1);
			int dif = (int)c1.dif(c2);
			printf("c_dif: %d", dif);
			if (dif < 200) {
				pxl1[0] = 255;
				pxl1[1] = 0;
				pxl1[2] = 0;
				pxl1[3] = 1;
			}
		}
	}
	return img1;
}

TargaImage * colorLimit(TargaImage * img){
	//std::vector<Color> colors;
	//Color c = Color(255, 0, 0, 1);
	//colors.push_back(c);
	//printf("Color: %d %d %d %d", c.r, c.g, c.b, c.a);
	
	std::vector<Color> colors;
	//colors.push_back(Color(255, 0, 0, 1));
	//colors.push_back(Color(0, 255, 0, 1));
	//colors.push_back(Color(0, 0, 255, 1));
	colors.push_back(Color(156, 120, 126, 1));
	colors.push_back(Color(128, 64, 64, 1));
	colors.push_back(Color(255, 255, 255, 1));	//white
	colors.push_back(Color(255, 222, 173, 1));	//navajo white
	colors.push_back(Color(238, 221, 130, 1));	//light goldenrod
	colors.push_back(Color(139, 69, 19, 1));	//saddle brown
	colors.push_back(Color(25, 25, 112, 1));	//navy blue
	colors.push_back(Color(64, 128, 128, 1));
	colors.push_back(Color(255, 30, 128, 1));
	colors.push_back(Color(0, 128, 0, 1));		//green
	
	//printf("Size: %d", colors.size());

	
	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			unsigned char * pxl = img->getPixel(x, y);
			Color pxl_c(pxl[0], pxl[1], pxl[2], pxl[3]);
			//printf("Pxl Color: %d %d %d %d", pxl_c.r, pxl_c.g, pxl_c.b, pxl_c.a);
			int min_c = 0;
			int min_err = pxl_c.dif(colors[0]);
			for(int c = 1; c < colors.size(); c++){
				int c_err = pxl_c.dif(colors[c]);
				if(c_err < min_err) {
					min_c = c;
					min_err = c_err;
				}
			}

			pxl[0] = colors.at(min_c).r;
			pxl[1] = colors.at(min_c).g;
			pxl[2] = colors.at(min_c).b;
			pxl[3] = colors.at(min_c).a;
		}
	}
	
	return img;
}

//returns a black and white version of the <image>
TargaImage* bw(TargaImage* image){
	float d_right[] = {	0,		0,		0,
						0,		0,	.4375,
					.1875,	.3125,	.0625};

	float d_left[] = {	0,		0,		0,
					.4375,		0,		0,
					.0625,	.3125,	.1875};
	
	//get new grey scaled version of image
	TargaImage* bw_image = greyscale(image);

	//run Floyd-Steinburg Algorithm
	for(int y=0; y < image->height; y++) {		
		if(y%2 == 0){
			for(int x=0; x < image->width; x++){
				FS_dither_pxl(bw_image, x, y, d_right, 3);
			}
		} else {
			for(int x=image->width-1; x >= 0; x--){
				FS_dither_pxl(bw_image, x, y, d_left, 3);
			}
		}
	}

	return bw_image;
}

//returns a greyscaled version of the <image>
TargaImage* greyscale(TargaImage* image){
	//create new image
	unsigned char* new_data = new unsigned char[image->width * image->height * 4];
	TargaImage* grey_image = new TargaImage(image->width, image->height, new_data);
	delete[] new_data;

	//greyscale the image
	for(int x=0; x < image->width; x++){
		for(int y=0; y < image->height; y++){
			unsigned char* pxl  = image->getPixel(x,y);
			unsigned char* pxl_grey = grey_image->getPixel(x, y);
				int L = .3 * pxl[0] + .59 * pxl[1] + .11 * pxl[2];	//Y = 0.30R + 0.59G + 0.11B
				pxl_grey[0]	= L;		// desaturate red
				pxl_grey[1]	= L;		// desaturate green
				pxl_grey[2]	= L;		// desaturate blue
				pxl_grey[3]	= pxl[3];	// leave alpha alone
		}
	}
	return grey_image;
}

/* places <brush> stroke of radius <brush_rad> on the <new_image> given the value of the corresponding
 * pixel <x>,<y> in the original <image>.
 */
void impress_pxl(TargaImage* image, TargaImage* new_image, int x, int y, float* brush, int brush_rad){
	unsigned char* pxl  = image->getPixel(x, y);
	for(int i = -brush_rad; i < brush_rad; i++){
		for(int j = -brush_rad; j < brush_rad; j++){
			unsigned char* new_pxl  = new_image->getPixel(x+i, y+j);
			if(new_pxl != NULL){
				new_pxl[0] = brush[c_pos(i, j, brush_rad)] * pxl[0] + (1 - brush[c_pos(i, j, brush_rad)]) * new_pxl[0];
				new_pxl[1] = brush[c_pos(i, j, brush_rad)] * pxl[1] + (1 - brush[c_pos(i, j, brush_rad)]) * new_pxl[1];
				new_pxl[2] = brush[c_pos(i, j, brush_rad)] * pxl[2] + (1 - brush[c_pos(i, j, brush_rad)]) * new_pxl[2];
			}
		}
	}
}

/* returns a painted verison of <image> using the selected brush
 */
TargaImage* impress(TargaImage* image, int brush){
	const int SC_BRUSH = 0;
	const int C_BRUSH  = 1;
	const int SP_BRUSH = 2;
	const int BS_BRUSH = 3;
	const int TL_BRUSH = 4;
	const int ALL_BRUSH = 5;
	int all_prob = 30;

	//small circle brush
	int sc_prob = 10;
	int sc_brush_rad = 3;
	float sc_brush[] =
					{	0,	0,	1,	1,	1,	0,	0,
						0,	1,	1,	1,	1,	1,	0,
						1,	1,	1,	1,	1,	1,	1,
						1,	1,	1,	1,	1,	1,	1,
						1,	1,	1,	1,	1,	1,	1,
						0,	1,	1,	1,	1,	1,	0,
						0,	0,	1,	1,	1,	0,	0};
	
	//cresent brush
	int c_prob = 6;
	int c_brush_rad = 3;
	float c_brush[] =
					{	0,	1,	1,	.3,	.2,	.1,	0,
						0,	1,	1,	0,	0,	0,	0,
						1,	1,	0,	0,	0,	0,	0,
						1,	1,	0,	0,	0,	0,	0,
						1,	1,	0,	0,	0,	0,	0,
						0,	1,	0,	0,	0,	0,	0,
						0,	1,	1,	1,	1,	.4,	0};
	//spike brush
	//cresent brush
	int sp_prob = 4;
	int sp_brush_rad = 3;
	float sp_brush[] =
					{	0,	0,	0,	0,	0,	1,	0,
						0,	0,	0,	0,	1,	0,	0,
						0,	0,	1,	1,	1,	0,	0,
						0,	1,	1,	1,	0,	0,	0,
						1,	1,	1,	1,	0,	0,	0,
						1,	1,	1,	0,	0,	0,	0,
						1,	1,	1,	0,	0,	0,	0};

	//big square brush
	int bs_prob = 500;
	int bs_brush_rad = 20;
	float *bs_brush = new float[1681];
	for(int i = 0; i < 1681; i++){
		bs_brush[i] = 1;
	}

	//three lines brush
	int tl_prob = 14;
	int tl_brush_rad = 7;
	float tl_brush[] =
					{	.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6, .6,	.5,	.4,	.4,	.3,	.2,	.1,
						.2,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	.2,	
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						.2,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	.2,
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						.2,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	.2,	
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1};

	//create new image
	unsigned char* new_data = new unsigned char[image->width * image->height * 4];
	TargaImage* new_image = new TargaImage(image->width, image->height, new_data);
	delete[] new_data;
	
	if(brush != ALL_BRUSH){
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				switch(brush){
					case SC_BRUSH:
						if(rand()%sc_prob == 0)	impress_pxl(image, new_image, x, y, sc_brush, sc_brush_rad);
						break;
					case C_BRUSH:
						if(rand()%c_prob == 0)	impress_pxl(image, new_image, x, y, c_brush, c_brush_rad);
						break;
					case SP_BRUSH:
						if(rand()%sp_prob == 0)	impress_pxl(image, new_image, x, y, sp_brush, sp_brush_rad);
						break;
					case BS_BRUSH:
						if(rand()%bs_prob == 0)	impress_pxl(image, new_image, x, y, bs_brush, bs_brush_rad);
						break;
					case TL_BRUSH:
						if(rand()%tl_prob == 0)	impress_pxl(image, new_image, x, y, tl_brush, tl_brush_rad);
						break;
					default:
						cout << "IMPRESSIONIST usage: p1 <input_image> <output_image> 2 <brush_number(0-5)>" << endl;
						exit(1);
				}
			}
		}
	} else {
		//run all brushes in series
		//big square
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(bs_prob*1000)) impress_pxl(image, new_image, x, y, bs_brush, bs_brush_rad);
			}
		}
		//small circle
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(sc_prob*20) == 0) impress_pxl(image, new_image, x, y, sc_brush, sc_brush_rad);
			}
		}
		//cresent
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(c_prob*20) == 0) impress_pxl(image, new_image, x, y, c_brush, c_brush_rad);
			}
		}
		//spike
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(sp_prob*20) == 0) impress_pxl(image, new_image, x, y, sp_brush, sp_brush_rad);
			}
		}
		//threee lines
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(tl_prob*20) == 0) impress_pxl(image, new_image, x, y, tl_brush, tl_brush_rad);
			}
		}
	}

	return new_image;
}

/* returns a number normalized to the range [-1, 1] given
 * a number in [0,max]
 */
double normalize(int x, int max){
	float MD2 = max/2.0;
	return x/MD2 - 1;
}

/* returns a number normalized to the range [0, max] given
 * a number in [-1,1]
 */
float denormalize(double x, int max){
	float MD2 = max/2.0;
	return (1 + x) * MD2;
}

//nl-transform on <x>,<y>
void warp0_pxl(TargaImage* image, float* x, float* y){
	int x_max = image->width - 1;
	int y_max = image->height - 1;
	double x_norm = normalize(*x, x_max);
	double y_norm = normalize(*y, y_max);
	double r = sqrt(x_norm*x_norm + y_norm*y_norm);
	double phi = atan2(y_norm, x_norm);
	
	*x = denormalize(asin(x_norm)/PID2, x_max);
	*y = denormalize(asin(y_norm)/PID2, y_max);
}

//nl-transform on <x>,<y>
void warp1_pxl(TargaImage* image, float* x, float* y){
	int x_max = image->width - 1;
	int y_max = image->height - 1;
	double x_norm = normalize(*x, x_max);
	double y_norm = normalize(*y, y_max);
	double r = sqrt(x_norm*x_norm + y_norm*y_norm);
	double phi = atan2(y_norm, x_norm);
	r = sqrt(r);

	*x = denormalize(r * cos(phi), x_max);
	*y = denormalize(r * sin(phi), y_max);
}

//nl-transform on <x>,<y>
void warp2_pxl(TargaImage* image, float* x, float* y){
	int x_max = image->width - 1;
	int y_max = image->height - 1;
	double x_norm = normalize(*x, x_max);
	double y_norm = normalize(*y, y_max);
	double r = sqrt(x_norm*x_norm + y_norm*y_norm);
	double phi = atan2(y_norm, x_norm);
	r = 2*asin(r)/PI;

	*x = denormalize(r * cos(phi), x_max);
	*y = denormalize(r * sin(phi), y_max);
}


/* returns a warped image using the non-linear transform described by <transform>
 * using <filter> for interpolation
*/
TargaImage* warp(TargaImage* image, void(*transform)(TargaImage*, float*, float*), int filter){
	//create new image
	unsigned char* new_data = new unsigned char[image->width * image->height * 4];
	TargaImage* new_image = new TargaImage(image->width, image->height, new_data);
	delete[] new_data;

	unsigned char* pxl;
	unsigned char* new_pxl;
	for(int y=0; y < new_image->height; y++){
		for(int x=0; x < new_image->width; x++){
			float u = x, v = y;
			transform(image, &u, &v);

			if(filter == 0){//no interpolation
				pxl = image->getPixel((int)u, (int)v);
				new_pxl = new_image->getPixel(x, y);
				if(pxl != NULL){
					copy_pxl(pxl, new_pxl);
				} else {
					new_pxl[0] = 0;
					new_pxl[1] = 0;
					new_pxl[2] = 0;
				}

			} else if(filter == 1){//fixed size interpolation radius of 3
				new_pxl = new_image->getPixel(x, y);
				convolute2d_pxl(image, new_pxl, u, v, &bspline_filter, 3, 3);
	
			} else if(filter == 2) {//variable size interpolation
				new_pxl = new_image->getPixel(x, y);

				float px_u = x - 1, px_v = y;
				float nx_u = x + 1, nx_v = y;
				float py_u = x, py_v = y - 1;
				float ny_u = x, ny_v = y + 1;

				transform(image, &px_u, &px_v);
				transform(image, &nx_u, &nx_v);
				transform(image, &py_u, &py_v);
				transform(image, &ny_u, &ny_v);

				float x_rad = abs(nx_u - px_u);
				float y_rad = abs(ny_v - py_v);

				if(x_rad < l) x_rad = l;
				if(y_rad < l) y_rad = l;
	
				//if(x_rad > 0 && y_rad > 0) cout << x_rad << " " << y_rad << " " ;
				convolute2d_pxl(image, new_pxl, u, v, &bspline_filter, x_rad, y_rad);
			} else {
				cout << "WARP usage: p1 <input_image> <output_image> 3 <warp_number(0-2)> <filter_number(0-2)>" << endl;
				exit(1);
			}
		}
	}

	return new_image;
}

TargaImage * HSL(TargaImage * img, bool setH, bool setS, bool setL){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * hsl_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	//printf("w x h: %d x %d : ", img->width, img->height);
	
	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			unsigned char* pxl = img->getPixel(x, y);
			double h, s, l;
			float r, g, b;
			r = pxl[0];
			g = pxl[1];
			b = pxl[2];
			
			//CONVERT TO HSL
			RGBtoHSL(r, g, b, &h, &s, &l);
			
			//MULTIPLE BY WEIGHTS
			if(setH) h = .5;
			if(setS) s = 1;
			if(setL) l = .5;
			
			//CONVERT TO BACK TO RGB
			HSLtoRGB(h, s, l, &r, &g, &b);
			hsl_image->setPixel(x, y, r, g, b);
			//hsl_image = resize(hsl_image, 800, 600, &bspline_filter);
			//printf("RGB1 %3.1f %3.1f %3.1f\n", r, g, b);
		}
	}
	
	return hsl_image;
}

//Snobel Edge Detector

TargaImage* SnobelEdge(TargaImage * img){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * edge_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;

	int	GX[3][3];
	int	GY[3][3];
	
	/* 3x3 GX Sobel mask.  Ref: www.cee.hw.ac.uk/hipr/html/sobel.html */
	GX[0][0] = -1; GX[0][1] = 0; GX[0][2] = 1;
	GX[1][0] = -2; GX[1][1] = 0; GX[1][2] = 2;
	GX[2][0] = -1; GX[2][1] = 0; GX[2][2] = 1;

	/* 3x3 GY Sobel mask.  Ref: www.cee.hw.ac.uk/hipr/html/sobel.html */
	GY[0][0] =  1; GY[0][1] =  2; GY[0][2] =  1;
	GY[1][0] =  0; GY[1][1] =  0; GY[1][2] =  0;
	GY[2][0] = -1; GY[2][1] = -2; GY[2][2] = -1;

	for(int y=0; y <=(img->height-1); y++)  {
		for(int x=0; x<=(img->width-1); x++)  {
			int SUM;
			int sumX = 0;
			int sumY = 0;

			// image boundaries
			if(y==0 || y==img->height-1){
				SUM = 0;
			} else if(x==0 || x==img->width-1){
				SUM = 0;
			// Convolution starts here
			} else {
				//-------X GRADIENT APPROXIMATION------
				for(int i=-1; i<=1; i++)  {
					for(int j=-1; j<=1; j++)  {
						sumX = sumX + (int)((img->getPixel(x + i, y + j))[0] * GX[i+1][j+1]);
					}
				}

				//-------Y GRADIENT APPROXIMATION-------
				for(int i=-1; i<=1; i++)  {
					for(int j=-1; j<=1; j++)  {
						sumY = sumY + (int)((img->getPixel(x + i, y + j))[0] * GY[i+1][j+1]);
					}
				}

				//---GRADIENT MAGNITUDE APPROXIMATION (Myler p.218)----
				SUM = abs(sumX) + abs(sumY);
			}

			if(SUM>255) SUM=255;
			if(SUM<0) SUM=0;

			edge_image->setPixel(x, y, 255 - (unsigned char)(SUM), 255 - (unsigned char)(SUM), 255 - (unsigned char)(SUM));
		}
	}
	return edge_image;
}

TargaImage * YCC(TargaImage * img){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * ycc_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	
	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			unsigned char* pxl = img->getPixel(x, y);
			double yc, cb, cr;
			float r, g, b;
			r = pxl[0];
			g = pxl[1];
			b = pxl[2];
			
			//check average to filter out white/grey
			float avgRGB = (r+g+b)/3.0;
			float white_prox = 15;
			bool notWhite = abs(r - avgRGB) > white_prox || 
							abs(g - avgRGB) > white_prox ||
							abs(b - avgRGB) > white_prox;

			RGBtoYCC(r, g, b, &yc, &cb, &cr);
			//if(cr > 140 && cr < 165 && cb > 105 && cb < 135){	//study norm
			if(cr > 120 && cr < 165 && cb > 150 && cb < 190 && notWhite){	//incandesent (home) 120 165 0 190
			//if(cr > 140 && cr < 165 && cb > 105 && cb < 200){	//floresent (work)
				ycc_image->setPixel(x, y, r, g, b);
			} else {
				ycc_image->setPixel(x, y, 0, 0, 0);
			}
			//YCCtoRGB(y, cb, cr, &r, &g, &b);
			
			
			//printf("RGB1 %3.1f %3.1f %3.1f\n", r, g, b);
		}
	}
	
	return ycc_image;
}

TargaImage* Invert(TargaImage * img){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * inv_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	
	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			float ir, ig, ib;
			unsigned char* pxl = img->getPixel(x, y);
			invertRGB(pxl[0], pxl[1], pxl[2], &ir, &ig, &ib);
			inv_image->setPixel(x, y, ir, ig, ib);
		}
	}

	return inv_image;
}

TargaImage * Cartoonify(TargaImage * img){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * cartoon_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	
	//GET IMAGE EDGES
	TargaImage * edge_img = greyscale(img);
	edge_img = SnobelEdge(edge_img);

	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			unsigned char* pxl = img->getPixel(x, y);
			double h, s, l;
			float r, g, b;
			r = pxl[0];
			g = pxl[1];
			b = pxl[2];
			
			//CONVERT TO HSL			
			RGBtoHSL(r, g, b, &h, &s, &l);
			
			//SET L TO EDGE STRENGTH
			l = (edge_img->getPixel(x, y))[0]/255.0;

			//CONVERT BACK TO RGB
			HSLtoRGB(h, s, l, &r, &g, &b);
			cartoon_image->setPixel(x, y, r, g, b);
		}
	}
	
	return cartoon_image;
}

TargaImage * specBeGone(TargaImage * img){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * no_spec_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	//printf("w x h: %d x %d : ", img->width, img->height);

	unsigned char * last_pxl = img->getPixel(0, 0);
	double lh, ls, ll;
	RGBtoHSL(last_pxl[0], last_pxl[1], last_pxl[2], &lh, &ls, &ll);

	for(int y = 0; y < img->height; y++){
		for(int x = 0; x < img->width; x++){
			unsigned char* pxl = img->getPixel(x, y);
			double h, s, l;
			float r, g, b;
			r = pxl[0];
			g = pxl[1];
			b = pxl[2];
			
			//CONVERT TO HSL
			RGBtoHSL(r, g, b, &h, &s, &l);
			
			//CHECK LUMINANCE
			if(l > 0.8) {	//IF ITS TOO HIGH SET IT TO LAST GOOD VALUE
				h = lh;
				s = ls;
				l = ll;
			} else {		//ELSE SAVE THIS AS LAST GOOD VALUE
				lh = h;
				ls = s;
				ll = l;
			}

			HSLtoRGB(h, s, l, &r, &g, &b);
			no_spec_image->setPixel(x, y, r, g, b);
		}
	}
	
	return no_spec_image;
}

/*
void ColorFix(int * rgb, int * fixed_rgb){
	//brown
	//red
	//orange
	//yellow
	//green
	//blue
	//violet
	//white
	//black

}
*/
TargaImage * BWThreshold(TargaImage * img, float threshold){
	unsigned char* pxl;

	for(int y = 0; y < img->height; y++){
		for(int x = 0; x < img->width; x++){
			pxl = img->getPixel(x, y);
			if((pxl[0] + pxl[1] + pxl[2])/3.0 < threshold){
				pxl[0] = 0;
				pxl[1] = 0;
				pxl[2] = 0;
			} else {
				pxl[0] = 255;
				pxl[1] = 255;
				pxl[2] = 255;
			}
		}
	}

	return img;
}

TargaImage * Lichtenstein(TargaImage * img){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * lic_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	
	int step = 10;
	int step_2 = step*step;
	int step_d2 = step/2;
	int radius = 4;
	int threshold = 150;
	int offset = step_d2;
	bool offset_flag = true;
	bool line_flag = false;

	int avg_rgb[3]; 

	//weights
	/*
	int[][] w = {{1, 1, 1},
				{1, 1, 1},
				{1, 1, 1}};
	*/
	//GET IMAGE EDGES
	TargaImage * edge_img = greyscale(img);
	edge_img = SnobelEdge(edge_img);
	BWThreshold(edge_img, threshold);

	
	unsigned char* pxl;
	unsigned char* edge_pxl;
	//unsigned char * histogram = new unsigned char [(img->width * img->height)/step][4];
	unsigned char* hist_pxl;
	for(int y = step; y < img->height; y+=step){
		offset_flag = !offset_flag;
		int x_start = step - ((offset_flag == true)?offset:0);
		//printf("offset_flag:%d offset: %d x_start: %d\n", offset_flag, offset, x_start);
		

		for(int x = x_start; x < img->width; x+=step){
			//printf("pxl(%d,%d) step:%d step_d2:%d step^2:%d\n", x, y, step, step_d2, step_2);
			//clear old average
			avg_rgb[0] = avg_rgb[1] = avg_rgb[2] = 0;

			//get average over area
			for(int i = -step_d2; i < step_d2; i++){
				for(int j = -step_d2; j < step_d2; j++){
					if(pxl = img->getPixel(x + i, y + j)){
						//add pixel value to sum
						avg_rgb[0] += pxl[0];
						avg_rgb[1] += pxl[1];
						avg_rgb[2] += pxl[2];

						//while we're looping check for a line
						edge_pxl = edge_img->getPixel(x + i, y + j);
						if((edge_pxl != NULL) && (edge_pxl[0] == 0)){
							line_flag = true;
						}
					}
				}
			}
			//average
			avg_rgb[0] /= step_2;
			avg_rgb[1] /= step_2;
			avg_rgb[2] /= step_2;
			
			//add to histogram
			//histogram[

			//kmeans

			//reassign "more standard" colors

			//set new values
			if(!line_flag){
				for(int i = -step_d2; i < step_d2; i++){
					for(int j = -step_d2; j < step_d2; j++){
						if(sqrt((float)(i*i + j*j)) < radius){					
							edge_img->setPixel(x + i, y + j, avg_rgb[0], avg_rgb[1], avg_rgb[2]);
						}
						
						/*
						//check threshholded edge pixel
						edge_pxl = edge_img->getPixel(x + i, y + j);
						if((edge_pxl != NULL) && (edge_pxl[0] != 0)){
							//take average color to fill circle
							if(sqrt((float)(i*i + j*j)) < radius){
								edge_img->setPixel(x + i, y + j, avg_rgb[0], avg_rgb[1], avg_rgb[2]);
							}
						}*/

					}
				}
			}
			line_flag = false;
		}
	}

	//delete[] histogram;

	return edge_img;
}